/*********************************************************
 *                                                       *
 * Frosty Trap                                           *
 *                                                       *
 * This is actually a classic SOR kernel                 *
 *                                                       *
 * Created by Brian Vinter, June 18th 1999               *
 *                                                       *
 * This is not an example of perfect Java code,          *
 * but is meant as a simple basis for generating         *
 * parallel versions.                                    *
 ********************************************************/

#include <math.h>
#include "graphicsScreen.h"
#include "StopWatch.h"

//This is the trap class, the strange design is because the code
//should be parallelizable in as many ways as possible without
//changing the basics.

//map we are building

double *trap_data;

#define HEIGHT (300)
#define WIDTH (300)


int  colortable[] = {WHITE,  // -273.15;-226.15
		     BLUE,   // -226.15;-179.15
		     CYAN,   // -179.15;-132.15
		     YELLOW, // -132.15;- 85.15
		     ORANGE, // - 85.15;- 38.15
		     GREEN,  // - 38.15;+  8.85
		     RED     // +  8.85;+ 55.85
};


int color(double data) {
  //We translate between -273.15 and 55.85
  data+=273.15;
  
  return colortable[(int)(data/47)];
}

void update()
{
  int x,y;

  
  for(y=1;y<HEIGHT-1;y++)
    for(x=1;x<WIDTH-1;x++)
      gs_plot(x,y,color(trap_data[y*WIDTH+x]));
  gs_update();
}

void solve()                                  //Simple SOR algorithm
{                                        
  double delta=1000.0;
  double old;
  int updatef=0, show_freq=100;
  int itt=0;
  int h=HEIGHT;
  int w=WIDTH;
  double epsilon=0.001*h*w;
  int x,y;
  
  gs_update();
  while(delta>epsilon) {
    itt+=1;
    delta=0.0;
    for(y=1;y<h-1;y++)
      for(x=1;x<w-1;x++){
	old=trap_data[y*WIDTH+x];
	trap_data[y*WIDTH+x]=0.2*(trap_data[y*WIDTH+x]+trap_data[(y-1)*WIDTH+x]+trap_data[(y+1)*WIDTH+x]+trap_data[y*WIDTH+x-1]+trap_data[y*WIDTH+x+1]);
	delta+=fabs(trap_data[y*WIDTH+x]-old);
      }
    updatef+=1;
    if (updatef==show_freq){
      update();
      updatef=0;
    }
  }
  gs_update();
}


void CreateTrap() {        //Our 'main' function

  int x,y,color;                               //Holds the result of slove
  
  for(y=0;y<HEIGHT-1;y++)           //Setup left side temp
    trap_data[y*WIDTH+0]=-273.15;
  
  for(y=0;y<HEIGHT-1;y++)           //Setup right side temp
    trap_data[y*WIDTH+WIDTH-1]=-273.15;
  
  for(x=0; x<WIDTH; x++)            //Setup top temp
    trap_data[0*WIDTH+x]=40.0;
  
  for(x=0; x<WIDTH; x++)            //Setup bottom temp
    trap_data[(HEIGHT-1)*WIDTH+x]=-273.15;
  
  solve();
}

int main(int argc, char **argv) {           //This is run by JVM
  char buf[256];
  int i;
  trap_data=(double *)malloc(sizeof(double)*HEIGHT*WIDTH);

  gs_init(WIDTH,HEIGHT);
  sw_init();
  sw_start();
  CreateTrap();                            //Call our main
  sw_stop();
  sw_timeString(buf);
  printf("Time taken: %s\n",buf);
  gs_exit();
}









