/*********************************************************
 *                                                       *
 * Frosty Trap                                           *
 *                                                       *
 * This is actually a classic SOR kernel                 *
 *                                                       *
 * Created by Brian Vinter, June 18th 1999               *
 *                                                       *
 * This is not an example of perfect Java code,          *
 * but is meant as a simple basis for generating         *
 * parallel versions.                                    *
 ********************************************************/


import java.awt.*;
import java.lang.Math;
import java.util.Date;

//This is the trap class, the strange design is because the code
//should be parallelizable in as many ways as possible without
//changing the basics.

class Trap 
{
  public int height, width;                  //These hold the size of the
                                             //map we are building

  graphicsScreen oGS;

  public double[][] data;

  private Color[] colortable = {Color.white,  // -273.15;-226.15
				Color.blue,   // -226.15;-179.15
				Color.cyan,   // -179.15;-132.15
				Color.yellow, // -132.15;- 85.15
				Color.orange, // - 85.15;- 38.15
				Color.green,  // - 38.15;+  8.85
				Color.red     // +  8.85;+ 55.85
  };


  Trap(graphicsScreen oGS)                            //Constructor, simply initiates
  {                                          //hight and width
    this.oGS=oGS;
    height = oGS.hight;
    width = oGS.width;
    data=new double[height][width];
  }
  
  Color color(double data) {
    //We translate between -273.15 and 55.85
    data+=273.15;
    
    return colortable[(int)(data/47)];
  }

  void update()
  {
    for(int y=1;y<height-1;y++)
      for(int x=1;x<width-1;x++)
	oGS.plot(x,y,color(data[y][x]));
  }
  
}


public class FrostyTrap          //This is our map generator
{
  static Trap T;                                       //Create a map

  
  static void solve()                                  //Simple SOR algorithm
  {                                        
    double delta=1000.0;
    double old;
    int update=0, show_freq=100;
    int itt=0;
    int h=T.height;
    int w=T.width;
    double epsilon=0.001*h*w;
    
    T.update();
    while(delta>epsilon) {
      itt+=1;
      delta=0.0;
      for(int y=1;y<h-1;y++)
	for(int x=1;x<w-1;x++){
	  old=T.data[y][x];
	  T.data[y][x]=0.2*(T.data[y][x]+T.data[y-1][x]+T.data[y+1][x]+T.data[y][x-1]+T.data[y][x+1]);
	  delta+=Math.abs(T.data[y][x]-old);
	}
      update+=1;
      if (update==show_freq){
	T.update();
	update=0;
      }
    }
    T.update();
  }
  

  public static void CreateTrap(graphicsScreen oGS) {        //Our 'main' function

    int color;                               //Holds the result of slove

    T = new Trap(oGS);                         //Create the new Trap

    for(int y=0;y<T.height-1;y++)           //Setup left side temp
	T.data[y][0]=-273.15;


    for(int y=0;y<T.height-1;y++)           //Setup right side temp
      T.data[y][T.width-1]=-273.15;

    for(int x=0; x<T.width; x++)            //Setup top temp
      T.data[0][x]=40.0;

    for(int x=0; x<T.width; x++)            //Setup bottom temp
      T.data[T.height-1][x]=-273.15;

    solve();
  }

  public static void main(String [] args) {           //This is run by JVM
    Date time = new Date();
    double start, stop;
    
    StopWatch st = new StopWatch();

    st.start();
    start=time.getTime();                    //Get starttime
    CreateTrap(new graphicsScreen(300,300));                            //Call our main
    time = new Date();                       
    st.stop();                     //Get endtime
    System.out.println("Time taken: " + st.timeString());
  }
}









